# Copyright 2019 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time
import unittest
import uuid

from kubernetes.e2e_test import base
from kubernetes.client import api_client

from . import DynamicClient
from .resource import ResourceInstance, ResourceField
from .exceptions import ResourceNotFoundError


def short_uuid():
    id = str(uuid.uuid4())
    return id[-12:]


class TestDynamicClient(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.config = base.get_e2e_configuration()

    def test_cluster_custom_resources(self):
        client = DynamicClient(api_client.ApiClient(configuration=self.config))

        with self.assertRaises(ResourceNotFoundError):
            changeme_api = client.resources.get(
                api_version='apps.example.com/v1', kind='ClusterChangeMe')

        crd_api = client.resources.get(
            api_version='apiextensions.k8s.io/v1beta1',
            kind='CustomResourceDefinition')
        name = 'clusterchangemes.apps.example.com'
        crd_manifest = {
            'apiVersion': 'apiextensions.k8s.io/v1beta1',
            'kind': 'CustomResourceDefinition',
            'metadata': {
                'name': name,
            },
            'spec': {
                'group': 'apps.example.com',
                'names': {
                    'kind': 'ClusterChangeMe',
                    'listKind': 'ClusterChangeMeList',
                    'plural': 'clusterchangemes',
                    'singular': 'clusterchangeme',
                },
                'scope': 'Cluster',
                'version': 'v1',
                'subresources': {
                    'status': {}
                }
            }
        }
        resp = crd_api.create(crd_manifest)

        self.assertEqual(name, resp.metadata.name)
        self.assertTrue(resp.status)

        resp = crd_api.get(
            name=name,
        )
        self.assertEqual(name, resp.metadata.name)
        self.assertTrue(resp.status)

        try:
            changeme_api = client.resources.get(
                api_version='apps.example.com/v1', kind='ClusterChangeMe')
        except ResourceNotFoundError:
            # Need to wait a sec for the discovery layer to get updated
            time.sleep(2)
        changeme_api = client.resources.get(
            api_version='apps.example.com/v1', kind='ClusterChangeMe')
        resp = changeme_api.get()
        self.assertEqual(resp.items, [])
        changeme_name = 'custom-resource' + short_uuid()
        changeme_manifest = {
            'apiVersion': 'apps.example.com/v1',
            'kind': 'ClusterChangeMe',
            'metadata': {
                'name': changeme_name,
            },
            'spec': {}
        }

        resp = changeme_api.create(body=changeme_manifest)
        self.assertEqual(resp.metadata.name, changeme_name)

        resp = changeme_api.get(name=changeme_name)
        self.assertEqual(resp.metadata.name, changeme_name)

        changeme_manifest['spec']['size'] = 3
        resp = changeme_api.patch(
            body=changeme_manifest,
            content_type='application/merge-patch+json'
        )
        self.assertEqual(resp.spec.size, 3)

        resp = changeme_api.get(name=changeme_name)
        self.assertEqual(resp.spec.size, 3)

        resp = changeme_api.get()
        self.assertEqual(len(resp.items), 1)

        resp = changeme_api.delete(
            name=changeme_name,
        )

        resp = changeme_api.get()
        self.assertEqual(len(resp.items), 0)

        resp = crd_api.delete(
            name=name,
        )

        time.sleep(2)
        client.resources.invalidate_cache()
        with self.assertRaises(ResourceNotFoundError):
            changeme_api = client.resources.get(
                api_version='apps.example.com/v1', kind='ClusterChangeMe')

    def test_async_namespaced_custom_resources(self):
        client = DynamicClient(api_client.ApiClient(configuration=self.config))

        with self.assertRaises(ResourceNotFoundError):
            changeme_api = client.resources.get(
                api_version='apps.example.com/v1', kind='ChangeMe')

        crd_api = client.resources.get(
            api_version='apiextensions.k8s.io/v1beta1',
            kind='CustomResourceDefinition')

        name = 'changemes.apps.example.com'

        crd_manifest = {
            'apiVersion': 'apiextensions.k8s.io/v1beta1',
            'kind': 'CustomResourceDefinition',
            'metadata': {
                'name': name,
            },
            'spec': {
                'group': 'apps.example.com',
                'names': {
                    'kind': 'ChangeMe',
                    'listKind': 'ChangeMeList',
                    'plural': 'changemes',
                    'singular': 'changeme',
                },
                'scope': 'Namespaced',
                'version': 'v1',
                'subresources': {
                    'status': {}
                }
            }
        }
        async_resp = crd_api.create(crd_manifest, async_req=True)

        self.assertEqual(name, async_resp.metadata.name)
        self.assertTrue(async_resp.status)

        async_resp = crd_api.get(
            name=name,
            async_req=True
        )
        self.assertEqual(name, async_resp.metadata.name)
        self.assertTrue(async_resp.status)

        try:
            changeme_api = client.resources.get(
                api_version='apps.example.com/v1', kind='ChangeMe')
        except ResourceNotFoundError:
            # Need to wait a sec for the discovery layer to get updated
            time.sleep(2)
            changeme_api = client.resources.get(
                api_version='apps.example.com/v1', kind='ChangeMe')

        async_resp = changeme_api.get(async_req=True)
        self.assertEqual(async_resp.items, [])

        changeme_name = 'custom-resource' + short_uuid()
        changeme_manifest = {
            'apiVersion': 'apps.example.com/v1',
            'kind': 'ChangeMe',
            'metadata': {
                'name': changeme_name,
            },
            'spec': {}
        }

        async_resp = changeme_api.create(body=changeme_manifest, namespace='default', async_req=True)
        self.assertEqual(async_resp.metadata.name, changeme_name)

        async_resp = changeme_api.get(name=changeme_name, namespace='default', async_req=True)
        self.assertEqual(async_resp.metadata.name, changeme_name)

        changeme_manifest['spec']['size'] = 3
        async_resp = changeme_api.patch(
            body=changeme_manifest,
            namespace='default',
            content_type='application/merge-patch+json',
            async_req=True
        )
        self.assertEqual(async_resp.spec.size, 3)

        async_resp = changeme_api.get(name=changeme_name, namespace='default', async_req=True)
        self.assertEqual(async_resp.spec.size, 3)

        async_resp = changeme_api.get(namespace='default', async_req=True)
        self.assertEqual(len(async_resp.items), 1)

        async_resp = changeme_api.get(async_req=True)
        self.assertEqual(len(async_resp.items), 1)

        async_resp = changeme_api.delete(
            name=changeme_name,
            namespace='default',
            async_req=True
        )

        async_resp = changeme_api.get(namespace='default', async_req=True)
        self.assertEqual(len(async_resp.items), 0)

        async_resp = changeme_api.get(async_req=True)
        self.assertEqual(len(async_resp.items), 0)

        async_resp = crd_api.delete(
            name=name,
            async_req=True
        )

        time.sleep(2)
        client.resources.invalidate_cache()
        with self.assertRaises(ResourceNotFoundError):
            changeme_api = client.resources.get(
                api_version='apps.example.com/v1', kind='ChangeMe')

    def test_namespaced_custom_resources(self):
        client = DynamicClient(api_client.ApiClient(configuration=self.config))

        with self.assertRaises(ResourceNotFoundError):
            changeme_api = client.resources.get(
                api_version='apps.example.com/v1', kind='ChangeMe')

        crd_api = client.resources.get(
            api_version='apiextensions.k8s.io/v1beta1',
            kind='CustomResourceDefinition')
        name = 'changemes.apps.example.com'
        crd_manifest = {
            'apiVersion': 'apiextensions.k8s.io/v1beta1',
            'kind': 'CustomResourceDefinition',
            'metadata': {
                'name': name,
            },
            'spec': {
                'group': 'apps.example.com',
                'names': {
                    'kind': 'ChangeMe',
                    'listKind': 'ChangeMeList',
                    'plural': 'changemes',
                    'singular': 'changeme',
                },
                'scope': 'Namespaced',
                'version': 'v1',
                'subresources': {
                    'status': {}
                }
            }
        }
        resp = crd_api.create(crd_manifest)

        self.assertEqual(name, resp.metadata.name)
        self.assertTrue(resp.status)

        resp = crd_api.get(
            name=name,
        )
        self.assertEqual(name, resp.metadata.name)
        self.assertTrue(resp.status)

        try:
            changeme_api = client.resources.get(
                api_version='apps.example.com/v1', kind='ChangeMe')
        except ResourceNotFoundError:
            # Need to wait a sec for the discovery layer to get updated
            time.sleep(2)
        changeme_api = client.resources.get(
            api_version='apps.example.com/v1', kind='ChangeMe')
        resp = changeme_api.get()
        self.assertEqual(resp.items, [])
        changeme_name = 'custom-resource' + short_uuid()
        changeme_manifest = {
            'apiVersion': 'apps.example.com/v1',
            'kind': 'ChangeMe',
            'metadata': {
                'name': changeme_name,
            },
            'spec': {}
        }

        resp = changeme_api.create(body=changeme_manifest, namespace='default')
        self.assertEqual(resp.metadata.name, changeme_name)

        resp = changeme_api.get(name=changeme_name, namespace='default')
        self.assertEqual(resp.metadata.name, changeme_name)

        changeme_manifest['spec']['size'] = 3
        resp = changeme_api.patch(
            body=changeme_manifest,
            namespace='default',
            content_type='application/merge-patch+json'
        )
        self.assertEqual(resp.spec.size, 3)

        resp = changeme_api.get(name=changeme_name, namespace='default')
        self.assertEqual(resp.spec.size, 3)

        resp = changeme_api.get(namespace='default')
        self.assertEqual(len(resp.items), 1)

        resp = changeme_api.get()
        self.assertEqual(len(resp.items), 1)

        resp = changeme_api.delete(
            name=changeme_name,
            namespace='default'
        )

        resp = changeme_api.get(namespace='default')
        self.assertEqual(len(resp.items), 0)

        resp = changeme_api.get()
        self.assertEqual(len(resp.items), 0)

        resp = crd_api.delete(
            name=name,
        )

        time.sleep(2)
        client.resources.invalidate_cache()
        with self.assertRaises(ResourceNotFoundError):
            changeme_api = client.resources.get(
                api_version='apps.example.com/v1', kind='ChangeMe')

    def test_service_apis(self):
        client = DynamicClient(api_client.ApiClient(configuration=self.config))
        api = client.resources.get(api_version='v1', kind='Service')

        name = 'frontend-' + short_uuid()
        service_manifest = {'apiVersion': 'v1',
                            'kind': 'Service',
                            'metadata': {'labels': {'name': name},
                                         'name': name,
                                         'resourceversion': 'v1'},
                            'spec': {'ports': [{'name': 'port',
                                                'port': 80,
                                                'protocol': 'TCP',
                                                'targetPort': 80}],
                                     'selector': {'name': name}}}

        resp = api.create(
            body=service_manifest,
            namespace='default'
        )
        self.assertEqual(name, resp.metadata.name)
        self.assertTrue(resp.status)

        resp = api.get(
            name=name,
            namespace='default'
        )
        self.assertEqual(name, resp.metadata.name)
        self.assertTrue(resp.status)

        service_manifest['spec']['ports'] = [{'name': 'new',
                                              'port': 8080,
                                              'protocol': 'TCP',
                                              'targetPort': 8080}]
        resp = api.patch(
            body=service_manifest,
            name=name,
            namespace='default'
        )
        self.assertEqual(2, len(resp.spec.ports))
        self.assertTrue(resp.status)

        resp = api.delete(
            name=name, body={},
            namespace='default'
        )

    def test_replication_controller_apis(self):
        client = DynamicClient(api_client.ApiClient(configuration=self.config))
        api = client.resources.get(
            api_version='v1', kind='ReplicationController')

        name = 'frontend-' + short_uuid()
        rc_manifest = {
            'apiVersion': 'v1',
            'kind': 'ReplicationController',
            'metadata': {'labels': {'name': name},
                         'name': name},
            'spec': {'replicas': 2,
                     'selector': {'name': name},
                     'template': {'metadata': {
                         'labels': {'name': name}},
                         'spec': {'containers': [{
                             'image': 'nginx',
                             'name': 'nginx',
                             'ports': [{'containerPort': 80,
                                        'protocol': 'TCP'}]}]}}}}

        resp = api.create(
            body=rc_manifest, namespace='default')
        self.assertEqual(name, resp.metadata.name)
        self.assertEqual(2, resp.spec.replicas)

        resp = api.get(
            name=name, namespace='default')
        self.assertEqual(name, resp.metadata.name)
        self.assertEqual(2, resp.spec.replicas)

        api.delete(
            name=name,
            namespace='default',
            propagation_policy='Background')

    def test_configmap_apis(self):
        client = DynamicClient(api_client.ApiClient(configuration=self.config))
        api = client.resources.get(api_version='v1', kind='ConfigMap')

        name = 'test-configmap-' + short_uuid()
        test_configmap = {
            "kind": "ConfigMap",
            "apiVersion": "v1",
            "metadata": {
                "name": name,
                "labels": {
                    "e2e-test": "true",
                },
            },
            "data": {
                "config.json": "{\"command\":\"/usr/bin/mysqld_safe\"}",
                "frontend.cnf": "[mysqld]\nbind-address = 10.0.0.3\n"
            }
        }

        resp = api.create(
            body=test_configmap, namespace='default'
        )
        self.assertEqual(name, resp.metadata.name)

        resp = api.get(
            name=name, namespace='default', label_selector="e2e-test=true")
        self.assertEqual(name, resp.metadata.name)

        count = 0
        for _ in client.watch(api, timeout=10, namespace="default", name=name):
            count += 1
        self.assertTrue(count > 0, msg="no events received for watch")

        test_configmap['data']['config.json'] = "{}"
        resp = api.patch(
            name=name, namespace='default', body=test_configmap)

        resp = api.delete(
            name=name, body={}, namespace='default')

        resp = api.get(
            namespace='default',
            pretty=True,
            label_selector="e2e-test=true")
        self.assertEqual([], resp.items)

    def test_node_apis(self):
        client = DynamicClient(api_client.ApiClient(configuration=self.config))
        api = client.resources.get(api_version='v1', kind='Node')

        for item in api.get().items:
            node = api.get(name=item.metadata.name)
            self.assertTrue(len(dict(node.metadata.labels)) > 0)

    # test_node_apis_partial_object_metadata lists all nodes in the cluster,
    # but only retrieves object metadata
    def test_node_apis_partial_object_metadata(self):
        client = DynamicClient(api_client.ApiClient(configuration=self.config))
        api = client.resources.get(api_version='v1', kind='Node')

        params = {
            'header_params': {
                'Accept': 'application/json;as=PartialObjectMetadataList;v=v1;g=meta.k8s.io'}}
        resp = api.get(**params)
        self.assertEqual('PartialObjectMetadataList', resp.kind)
        self.assertEqual('meta.k8s.io/v1', resp.apiVersion)

        params = {
            'header_params': {
                'aCcePt': 'application/json;as=PartialObjectMetadataList;v=v1;g=meta.k8s.io'}}
        resp = api.get(**params)
        self.assertEqual('PartialObjectMetadataList', resp.kind)
        self.assertEqual('meta.k8s.io/v1', resp.apiVersion)

    def test_server_side_apply_api(self):
        client = DynamicClient(api_client.ApiClient(configuration=self.config))
        api = client.resources.get(
            api_version='v1', kind='Pod')

        name = 'pod-' + short_uuid()
        pod_manifest = {
                'apiVersion': 'v1',
                'kind': 'Pod',
                'metadata': {'labels': {'name': name},
                            'name': name},
                'spec': {'containers': [{
                            'image': 'nginx',
                            'name': 'nginx',
                            'ports': [{'containerPort': 80,
                                            'protocol': 'TCP'}]}]}}

        resp = api.server_side_apply(
            namespace='default', body=pod_manifest,
            field_manager='kubernetes-unittests', dry_run="All")
        self.assertEqual('kubernetes-unittests', resp.metadata.managedFields[0].manager)


class TestDynamicClientSerialization(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        config = base.get_e2e_configuration()
        cls.client = DynamicClient(api_client.ApiClient(configuration=config))
        cls.pod_manifest = {
            'apiVersion': 'v1',
            'kind': 'Pod',
            'metadata': {'name': 'foo-pod'},
            'spec': {'containers': [{'name': "main", 'image': "busybox"}]},
        }

    def test_dict_type(self):
        self.assertEqual(self.client.serialize_body(self.pod_manifest), self.pod_manifest)

    def test_resource_instance_type(self):
        inst = ResourceInstance(self.client, self.pod_manifest)
        self.assertEqual(self.client.serialize_body(inst), self.pod_manifest)

    def test_resource_field(self):
        """`ResourceField` is a special type which overwrites `__getattr__` method to return `None`
        when a non-existent attribute was accessed. which means it can pass any `hasattr(...)` tests.
        """
        params = {
            "foo": "bar",
            "self": True
        }
        res = ResourceField(params=params)
        self.assertEqual(res["foo"], params["foo"])
        self.assertEqual(res["self"], params["self"])
        self.assertEqual(self.client.serialize_body(res), params)



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: release-1.35
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kubernetes.client.configuration import Configuration


class V1EndpointSubset(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'addresses': 'list[V1EndpointAddress]',
        'not_ready_addresses': 'list[V1EndpointAddress]',
        'ports': 'list[CoreV1EndpointPort]'
    }

    attribute_map = {
        'addresses': 'addresses',
        'not_ready_addresses': 'notReadyAddresses',
        'ports': 'ports'
    }

    def __init__(self, addresses=None, not_ready_addresses=None, ports=None, local_vars_configuration=None):  # noqa: E501
        """V1EndpointSubset - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._addresses = None
        self._not_ready_addresses = None
        self._ports = None
        self.discriminator = None

        if addresses is not None:
            self.addresses = addresses
        if not_ready_addresses is not None:
            self.not_ready_addresses = not_ready_addresses
        if ports is not None:
            self.ports = ports

    @property
    def addresses(self):
        """Gets the addresses of this V1EndpointSubset.  # noqa: E501

        IP addresses which offer the related ports that are marked as ready. These endpoints should be considered safe for load balancers and clients to utilize.  # noqa: E501

        :return: The addresses of this V1EndpointSubset.  # noqa: E501
        :rtype: list[V1EndpointAddress]
        """
        return self._addresses

    @addresses.setter
    def addresses(self, addresses):
        """Sets the addresses of this V1EndpointSubset.

        IP addresses which offer the related ports that are marked as ready. These endpoints should be considered safe for load balancers and clients to utilize.  # noqa: E501

        :param addresses: The addresses of this V1EndpointSubset.  # noqa: E501
        :type: list[V1EndpointAddress]
        """

        self._addresses = addresses

    @property
    def not_ready_addresses(self):
        """Gets the not_ready_addresses of this V1EndpointSubset.  # noqa: E501

        IP addresses which offer the related ports but are not currently marked as ready because they have not yet finished starting, have recently failed a readiness check, or have recently failed a liveness check.  # noqa: E501

        :return: The not_ready_addresses of this V1EndpointSubset.  # noqa: E501
        :rtype: list[V1EndpointAddress]
        """
        return self._not_ready_addresses

    @not_ready_addresses.setter
    def not_ready_addresses(self, not_ready_addresses):
        """Sets the not_ready_addresses of this V1EndpointSubset.

        IP addresses which offer the related ports but are not currently marked as ready because they have not yet finished starting, have recently failed a readiness check, or have recently failed a liveness check.  # noqa: E501

        :param not_ready_addresses: The not_ready_addresses of this V1EndpointSubset.  # noqa: E501
        :type: list[V1EndpointAddress]
        """

        self._not_ready_addresses = not_ready_addresses

    @property
    def ports(self):
        """Gets the ports of this V1EndpointSubset.  # noqa: E501

        Port numbers available on the related IP addresses.  # noqa: E501

        :return: The ports of this V1EndpointSubset.  # noqa: E501
        :rtype: list[CoreV1EndpointPort]
        """
        return self._ports

    @ports.setter
    def ports(self, ports):
        """Sets the ports of this V1EndpointSubset.

        Port numbers available on the related IP addresses.  # noqa: E501

        :param ports: The ports of this V1EndpointSubset.  # noqa: E501
        :type: list[CoreV1EndpointPort]
        """

        self._ports = ports

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1EndpointSubset):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1EndpointSubset):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: release-1.35
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kubernetes.client.configuration import Configuration


class V1DeviceConstraint(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'distinct_attribute': 'str',
        'match_attribute': 'str',
        'requests': 'list[str]'
    }

    attribute_map = {
        'distinct_attribute': 'distinctAttribute',
        'match_attribute': 'matchAttribute',
        'requests': 'requests'
    }

    def __init__(self, distinct_attribute=None, match_attribute=None, requests=None, local_vars_configuration=None):  # noqa: E501
        """V1DeviceConstraint - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._distinct_attribute = None
        self._match_attribute = None
        self._requests = None
        self.discriminator = None

        if distinct_attribute is not None:
            self.distinct_attribute = distinct_attribute
        if match_attribute is not None:
            self.match_attribute = match_attribute
        if requests is not None:
            self.requests = requests

    @property
    def distinct_attribute(self):
        """Gets the distinct_attribute of this V1DeviceConstraint.  # noqa: E501

        DistinctAttribute requires that all devices in question have this attribute and that its type and value are unique across those devices.  This acts as the inverse of MatchAttribute.  This constraint is used to avoid allocating multiple requests to the same device by ensuring attribute-level differentiation.  This is useful for scenarios where resource requests must be fulfilled by separate physical devices. For example, a container requests two network interfaces that must be allocated from two different physical NICs.  # noqa: E501

        :return: The distinct_attribute of this V1DeviceConstraint.  # noqa: E501
        :rtype: str
        """
        return self._distinct_attribute

    @distinct_attribute.setter
    def distinct_attribute(self, distinct_attribute):
        """Sets the distinct_attribute of this V1DeviceConstraint.

        DistinctAttribute requires that all devices in question have this attribute and that its type and value are unique across those devices.  This acts as the inverse of MatchAttribute.  This constraint is used to avoid allocating multiple requests to the same device by ensuring attribute-level differentiation.  This is useful for scenarios where resource requests must be fulfilled by separate physical devices. For example, a container requests two network interfaces that must be allocated from two different physical NICs.  # noqa: E501

        :param distinct_attribute: The distinct_attribute of this V1DeviceConstraint.  # noqa: E501
        :type: str
        """

        self._distinct_attribute = distinct_attribute

    @property
    def match_attribute(self):
        """Gets the match_attribute of this V1DeviceConstraint.  # noqa: E501

        MatchAttribute requires that all devices in question have this attribute and that its type and value are the same across those devices.  For example, if you specified \"dra.example.com/numa\" (a hypothetical example!), then only devices in the same NUMA node will be chosen. A device which does not have that attribute will not be chosen. All devices should use a value of the same type for this attribute because that is part of its specification, but if one device doesn't, then it also will not be chosen.  Must include the domain qualifier.  # noqa: E501

        :return: The match_attribute of this V1DeviceConstraint.  # noqa: E501
        :rtype: str
        """
        return self._match_attribute

    @match_attribute.setter
    def match_attribute(self, match_attribute):
        """Sets the match_attribute of this V1DeviceConstraint.

        MatchAttribute requires that all devices in question have this attribute and that its type and value are the same across those devices.  For example, if you specified \"dra.example.com/numa\" (a hypothetical example!), then only devices in the same NUMA node will be chosen. A device which does not have that attribute will not be chosen. All devices should use a value of the same type for this attribute because that is part of its specification, but if one device doesn't, then it also will not be chosen.  Must include the domain qualifier.  # noqa: E501

        :param match_attribute: The match_attribute of this V1DeviceConstraint.  # noqa: E501
        :type: str
        """

        self._match_attribute = match_attribute

    @property
    def requests(self):
        """Gets the requests of this V1DeviceConstraint.  # noqa: E501

        Requests is a list of the one or more requests in this claim which must co-satisfy this constraint. If a request is fulfilled by multiple devices, then all of the devices must satisfy the constraint. If this is not specified, this constraint applies to all requests in this claim.  References to subrequests must include the name of the main request and may include the subrequest using the format <main request>[/<subrequest>]. If just the main request is given, the constraint applies to all subrequests.  # noqa: E501

        :return: The requests of this V1DeviceConstraint.  # noqa: E501
        :rtype: list[str]
        """
        return self._requests

    @requests.setter
    def requests(self, requests):
        """Sets the requests of this V1DeviceConstraint.

        Requests is a list of the one or more requests in this claim which must co-satisfy this constraint. If a request is fulfilled by multiple devices, then all of the devices must satisfy the constraint. If this is not specified, this constraint applies to all requests in this claim.  References to subrequests must include the name of the main request and may include the subrequest using the format <main request>[/<subrequest>]. If just the main request is given, the constraint applies to all subrequests.  # noqa: E501

        :param requests: The requests of this V1DeviceConstraint.  # noqa: E501
        :type: list[str]
        """

        self._requests = requests

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1DeviceConstraint):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1DeviceConstraint):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: release-1.35
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kubernetes.client.configuration import Configuration


class V1PodCertificateProjection(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'certificate_chain_path': 'str',
        'credential_bundle_path': 'str',
        'key_path': 'str',
        'key_type': 'str',
        'max_expiration_seconds': 'int',
        'signer_name': 'str',
        'user_annotations': 'dict(str, str)'
    }

    attribute_map = {
        'certificate_chain_path': 'certificateChainPath',
        'credential_bundle_path': 'credentialBundlePath',
        'key_path': 'keyPath',
        'key_type': 'keyType',
        'max_expiration_seconds': 'maxExpirationSeconds',
        'signer_name': 'signerName',
        'user_annotations': 'userAnnotations'
    }

    def __init__(self, certificate_chain_path=None, credential_bundle_path=None, key_path=None, key_type=None, max_expiration_seconds=None, signer_name=None, user_annotations=None, local_vars_configuration=None):  # noqa: E501
        """V1PodCertificateProjection - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._certificate_chain_path = None
        self._credential_bundle_path = None
        self._key_path = None
        self._key_type = None
        self._max_expiration_seconds = None
        self._signer_name = None
        self._user_annotations = None
        self.discriminator = None

        if certificate_chain_path is not None:
            self.certificate_chain_path = certificate_chain_path
        if credential_bundle_path is not None:
            self.credential_bundle_path = credential_bundle_path
        if key_path is not None:
            self.key_path = key_path
        self.key_type = key_type
        if max_expiration_seconds is not None:
            self.max_expiration_seconds = max_expiration_seconds
        self.signer_name = signer_name
        if user_annotations is not None:
            self.user_annotations = user_annotations

    @property
    def certificate_chain_path(self):
        """Gets the certificate_chain_path of this V1PodCertificateProjection.  # noqa: E501

        Write the certificate chain at this path in the projected volume.  Most applications should use credentialBundlePath.  When using keyPath and certificateChainPath, your application needs to check that the key and leaf certificate are consistent, because it is possible to read the files mid-rotation.  # noqa: E501

        :return: The certificate_chain_path of this V1PodCertificateProjection.  # noqa: E501
        :rtype: str
        """
        return self._certificate_chain_path

    @certificate_chain_path.setter
    def certificate_chain_path(self, certificate_chain_path):
        """Sets the certificate_chain_path of this V1PodCertificateProjection.

        Write the certificate chain at this path in the projected volume.  Most applications should use credentialBundlePath.  When using keyPath and certificateChainPath, your application needs to check that the key and leaf certificate are consistent, because it is possible to read the files mid-rotation.  # noqa: E501

        :param certificate_chain_path: The certificate_chain_path of this V1PodCertificateProjection.  # noqa: E501
        :type: str
        """

        self._certificate_chain_path = certificate_chain_path

    @property
    def credential_bundle_path(self):
        """Gets the credential_bundle_path of this V1PodCertificateProjection.  # noqa: E501

        Write the credential bundle at this path in the projected volume.  The credential bundle is a single file that contains multiple PEM blocks. The first PEM block is a PRIVATE KEY block, containing a PKCS#8 private key.  The remaining blocks are CERTIFICATE blocks, containing the issued certificate chain from the signer (leaf and any intermediates).  Using credentialBundlePath lets your Pod's application code make a single atomic read that retrieves a consistent key and certificate chain.  If you project them to separate files, your application code will need to additionally check that the leaf certificate was issued to the key.  # noqa: E501

        :return: The credential_bundle_path of this V1PodCertificateProjection.  # noqa: E501
        :rtype: str
        """
        return self._credential_bundle_path

    @credential_bundle_path.setter
    def credential_bundle_path(self, credential_bundle_path):
        """Sets the credential_bundle_path of this V1PodCertificateProjection.

        Write the credential bundle at this path in the projected volume.  The credential bundle is a single file that contains multiple PEM blocks. The first PEM block is a PRIVATE KEY block, containing a PKCS#8 private key.  The remaining blocks are CERTIFICATE blocks, containing the issued certificate chain from the signer (leaf and any intermediates).  Using credentialBundlePath lets your Pod's application code make a single atomic read that retrieves a consistent key and certificate chain.  If you project them to separate files, your application code will need to additionally check that the leaf certificate was issued to the key.  # noqa: E501

        :param credential_bundle_path: The credential_bundle_path of this V1PodCertificateProjection.  # noqa: E501
        :type: str
        """

        self._credential_bundle_path = credential_bundle_path

    @property
    def key_path(self):
        """Gets the key_path of this V1PodCertificateProjection.  # noqa: E501

        Write the key at this path in the projected volume.  Most applications should use credentialBundlePath.  When using keyPath and certificateChainPath, your application needs to check that the key and leaf certificate are consistent, because it is possible to read the files mid-rotation.  # noqa: E501

        :return: The key_path of this V1PodCertificateProjection.  # noqa: E501
        :rtype: str
        """
        return self._key_path

    @key_path.setter
    def key_path(self, key_path):
        """Sets the key_path of this V1PodCertificateProjection.

        Write the key at this path in the projected volume.  Most applications should use credentialBundlePath.  When using keyPath and certificateChainPath, your application needs to check that the key and leaf certificate are consistent, because it is possible to read the files mid-rotation.  # noqa: E501

        :param key_path: The key_path of this V1PodCertificateProjection.  # noqa: E501
        :type: str
        """

        self._key_path = key_path

    @property
    def key_type(self):
        """Gets the key_type of this V1PodCertificateProjection.  # noqa: E501

        The type of keypair Kubelet will generate for the pod.  Valid values are \"RSA3072\", \"RSA4096\", \"ECDSAP256\", \"ECDSAP384\", \"ECDSAP521\", and \"ED25519\".  # noqa: E501

        :return: The key_type of this V1PodCertificateProjection.  # noqa: E501
        :rtype: str
        """
        return self._key_type

    @key_type.setter
    def key_type(self, key_type):
        """Sets the key_type of this V1PodCertificateProjection.

        The type of keypair Kubelet will generate for the pod.  Valid values are \"RSA3072\", \"RSA4096\", \"ECDSAP256\", \"ECDSAP384\", \"ECDSAP521\", and \"ED25519\".  # noqa: E501

        :param key_type: The key_type of this V1PodCertificateProjection.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and key_type is None:  # noqa: E501
            raise ValueError("Invalid value for `key_type`, must not be `None`")  # noqa: E501

        self._key_type = key_type

    @property
    def max_expiration_seconds(self):
        """Gets the max_expiration_seconds of this V1PodCertificateProjection.  # noqa: E501

        maxExpirationSeconds is the maximum lifetime permitted for the certificate.  Kubelet copies this value verbatim into the PodCertificateRequests it generates for this projection.  If omitted, kube-apiserver will set it to 86400(24 hours). kube-apiserver will reject values shorter than 3600 (1 hour).  The maximum allowable value is 7862400 (91 days).  The signer implementation is then free to issue a certificate with any lifetime *shorter* than MaxExpirationSeconds, but no shorter than 3600 seconds (1 hour).  This constraint is enforced by kube-apiserver. `kubernetes.io` signers will never issue certificates with a lifetime longer than 24 hours.  # noqa: E501

        :return: The max_expiration_seconds of this V1PodCertificateProjection.  # noqa: E501
        :rtype: int
        """
        return self._max_expiration_seconds

    @max_expiration_seconds.setter
    def max_expiration_seconds(self, max_expiration_seconds):
        """Sets the max_expiration_seconds of this V1PodCertificateProjection.

        maxExpirationSeconds is the maximum lifetime permitted for the certificate.  Kubelet copies this value verbatim into the PodCertificateRequests it generates for this projection.  If omitted, kube-apiserver will set it to 86400(24 hours). kube-apiserver will reject values shorter than 3600 (1 hour).  The maximum allowable value is 7862400 (91 days).  The signer implementation is then free to issue a certificate with any lifetime *shorter* than MaxExpirationSeconds, but no shorter than 3600 seconds (1 hour).  This constraint is enforced by kube-apiserver. `kubernetes.io` signers will never issue certificates with a lifetime longer than 24 hours.  # noqa: E501

        :param max_expiration_seconds: The max_expiration_seconds of this V1PodCertificateProjection.  # noqa: E501
        :type: int
        """

        self._max_expiration_seconds = max_expiration_seconds

    @property
    def signer_name(self):
        """Gets the signer_name of this V1PodCertificateProjection.  # noqa: E501

        Kubelet's generated CSRs will be addressed to this signer.  # noqa: E501

        :return: The signer_name of this V1PodCertificateProjection.  # noqa: E501
        :rtype: str
        """
        return self._signer_name

    @signer_name.setter
    def signer_name(self, signer_name):
        """Sets the signer_name of this V1PodCertificateProjection.

        Kubelet's generated CSRs will be addressed to this signer.  # noqa: E501

        :param signer_name: The signer_name of this V1PodCertificateProjection.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and signer_name is None:  # noqa: E501
            raise ValueError("Invalid value for `signer_name`, must not be `None`")  # noqa: E501

        self._signer_name = signer_name

    @property
    def user_annotations(self):
        """Gets the user_annotations of this V1PodCertificateProjection.  # noqa: E501

        userAnnotations allow pod authors to pass additional information to the signer implementation.  Kubernetes does not restrict or validate this metadata in any way.  These values are copied verbatim into the `spec.unverifiedUserAnnotations` field of the PodCertificateRequest objects that Kubelet creates.  Entries are subject to the same validation as object metadata annotations, with the addition that all keys must be domain-prefixed. No restrictions are placed on values, except an overall size limitation on the entire field.  Signers should document the keys and values they support. Signers should deny requests that contain keys they do not recognize.  # noqa: E501

        :return: The user_annotations of this V1PodCertificateProjection.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._user_annotations

    @user_annotations.setter
    def user_annotations(self, user_annotations):
        """Sets the user_annotations of this V1PodCertificateProjection.

        userAnnotations allow pod authors to pass additional information to the signer implementation.  Kubernetes does not restrict or validate this metadata in any way.  These values are copied verbatim into the `spec.unverifiedUserAnnotations` field of the PodCertificateRequest objects that Kubelet creates.  Entries are subject to the same validation as object metadata annotations, with the addition that all keys must be domain-prefixed. No restrictions are placed on values, except an overall size limitation on the entire field.  Signers should document the keys and values they support. Signers should deny requests that contain keys they do not recognize.  # noqa: E501

        :param user_annotations: The user_annotations of this V1PodCertificateProjection.  # noqa: E501
        :type: dict(str, str)
        """

        self._user_annotations = user_annotations

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1PodCertificateProjection):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1PodCertificateProjection):
            return True

        return self.to_dict() != other.to_dict()



# Copyright 2021 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import logging
logger = logging.getLogger("leaderelection")


class Config:
    # Validate config, exit if an error is detected
    def __init__(self, lock, lease_duration, renew_deadline, retry_period, onstarted_leading, onstopped_leading):
        self.jitter_factor = 1.2

        if lock is None:
            sys.exit("lock cannot be None")
        self.lock = lock

        if lease_duration <= renew_deadline:
            sys.exit("lease_duration must be greater than renew_deadline")

        if renew_deadline <= self.jitter_factor * retry_period:
            sys.exit("renewDeadline must be greater than retry_period*jitter_factor")

        if lease_duration < 1:
            sys.exit("lease_duration must be greater than one")

        if renew_deadline < 1:
            sys.exit("renew_deadline must be greater than one")

        if retry_period < 1:
            sys.exit("retry_period must be greater than one")

        self.lease_duration = lease_duration
        self.renew_deadline = renew_deadline
        self.retry_period = retry_period

        if onstarted_leading is None:
            sys.exit("callback onstarted_leading cannot be None")
        self.onstarted_leading = onstarted_leading

        if onstopped_leading is None:
            self.onstopped_leading = self.on_stoppedleading_callback
        else:
            self.onstopped_leading = onstopped_leading

    # Default callback for when the current candidate if a leader, stops leading
    def on_stoppedleading_callback(self):
        logger.info("stopped leading".format(self.lock.identity))



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: release-1.35
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kubernetes.client.configuration import Configuration


class V1beta1BasicDevice(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'all_nodes': 'bool',
        'allow_multiple_allocations': 'bool',
        'attributes': 'dict(str, V1beta1DeviceAttribute)',
        'binding_conditions': 'list[str]',
        'binding_failure_conditions': 'list[str]',
        'binds_to_node': 'bool',
        'capacity': 'dict(str, V1beta1DeviceCapacity)',
        'consumes_counters': 'list[V1beta1DeviceCounterConsumption]',
        'node_name': 'str',
        'node_selector': 'V1NodeSelector',
        'taints': 'list[V1beta1DeviceTaint]'
    }

    attribute_map = {
        'all_nodes': 'allNodes',
        'allow_multiple_allocations': 'allowMultipleAllocations',
        'attributes': 'attributes',
        'binding_conditions': 'bindingConditions',
        'binding_failure_conditions': 'bindingFailureConditions',
        'binds_to_node': 'bindsToNode',
        'capacity': 'capacity',
        'consumes_counters': 'consumesCounters',
        'node_name': 'nodeName',
        'node_selector': 'nodeSelector',
        'taints': 'taints'
    }

    def __init__(self, all_nodes=None, allow_multiple_allocations=None, attributes=None, binding_conditions=None, binding_failure_conditions=None, binds_to_node=None, capacity=None, consumes_counters=None, node_name=None, node_selector=None, taints=None, local_vars_configuration=None):  # noqa: E501
        """V1beta1BasicDevice - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._all_nodes = None
        self._allow_multiple_allocations = None
        self._attributes = None
        self._binding_conditions = None
        self._binding_failure_conditions = None
        self._binds_to_node = None
        self._capacity = None
        self._consumes_counters = None
        self._node_name = None
        self._node_selector = None
        self._taints = None
        self.discriminator = None

        if all_nodes is not None:
            self.all_nodes = all_nodes
        if allow_multiple_allocations is not None:
            self.allow_multiple_allocations = allow_multiple_allocations
        if attributes is not None:
            self.attributes = attributes
        if binding_conditions is not None:
            self.binding_conditions = binding_conditions
        if binding_failure_conditions is not None:
            self.binding_failure_conditions = binding_failure_conditions
        if binds_to_node is not None:
            self.binds_to_node = binds_to_node
        if capacity is not None:
            self.capacity = capacity
        if consumes_counters is not None:
            self.consumes_counters = consumes_counters
        if node_name is not None:
            self.node_name = node_name
        if node_selector is not None:
            self.node_selector = node_selector
        if taints is not None:
            self.taints = taints

    @property
    def all_nodes(self):
        """Gets the all_nodes of this V1beta1BasicDevice.  # noqa: E501

        AllNodes indicates that all nodes have access to the device.  Must only be set if Spec.PerDeviceNodeSelection is set to true. At most one of NodeName, NodeSelector and AllNodes can be set.  # noqa: E501

        :return: The all_nodes of this V1beta1BasicDevice.  # noqa: E501
        :rtype: bool
        """
        return self._all_nodes

    @all_nodes.setter
    def all_nodes(self, all_nodes):
        """Sets the all_nodes of this V1beta1BasicDevice.

        AllNodes indicates that all nodes have access to the device.  Must only be set if Spec.PerDeviceNodeSelection is set to true. At most one of NodeName, NodeSelector and AllNodes can be set.  # noqa: E501

        :param all_nodes: The all_nodes of this V1beta1BasicDevice.  # noqa: E501
        :type: bool
        """

        self._all_nodes = all_nodes

    @property
    def allow_multiple_allocations(self):
        """Gets the allow_multiple_allocations of this V1beta1BasicDevice.  # noqa: E501

        AllowMultipleAllocations marks whether the device is allowed to be allocated to multiple DeviceRequests.  If AllowMultipleAllocations is set to true, the device can be allocated more than once, and all of its capacity is consumable, regardless of whether the requestPolicy is defined or not.  # noqa: E501

        :return: The allow_multiple_allocations of this V1beta1BasicDevice.  # noqa: E501
        :rtype: bool
        """
        return self._allow_multiple_allocations

    @allow_multiple_allocations.setter
    def allow_multiple_allocations(self, allow_multiple_allocations):
        """Sets the allow_multiple_allocations of this V1beta1BasicDevice.

        AllowMultipleAllocations marks whether the device is allowed to be allocated to multiple DeviceRequests.  If AllowMultipleAllocations is set to true, the device can be allocated more than once, and all of its capacity is consumable, regardless of whether the requestPolicy is defined or not.  # noqa: E501

        :param allow_multiple_allocations: The allow_multiple_allocations of this V1beta1BasicDevice.  # noqa: E501
        :type: bool
        """

        self._allow_multiple_allocations = allow_multiple_allocations

    @property
    def attributes(self):
        """Gets the attributes of this V1beta1BasicDevice.  # noqa: E501

        Attributes defines the set of attributes for this device. The name of each attribute must be unique in that set.  The maximum number of attributes and capacities combined is 32.  # noqa: E501

        :return: The attributes of this V1beta1BasicDevice.  # noqa: E501
        :rtype: dict(str, V1beta1DeviceAttribute)
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """Sets the attributes of this V1beta1BasicDevice.

        Attributes defines the set of attributes for this device. The name of each attribute must be unique in that set.  The maximum number of attributes and capacities combined is 32.  # noqa: E501

        :param attributes: The attributes of this V1beta1BasicDevice.  # noqa: E501
        :type: dict(str, V1beta1DeviceAttribute)
        """

        self._attributes = attributes

    @property
    def binding_conditions(self):
        """Gets the binding_conditions of this V1beta1BasicDevice.  # noqa: E501

        BindingConditions defines the conditions for proceeding with binding. All of these conditions must be set in the per-device status conditions with a value of True to proceed with binding the pod to the node while scheduling the pod.  The maximum number of binding conditions is 4.  The conditions must be a valid condition type string.  This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.  # noqa: E501

        :return: The binding_conditions of this V1beta1BasicDevice.  # noqa: E501
        :rtype: list[str]
        """
        return self._binding_conditions

    @binding_conditions.setter
    def binding_conditions(self, binding_conditions):
        """Sets the binding_conditions of this V1beta1BasicDevice.

        BindingConditions defines the conditions for proceeding with binding. All of these conditions must be set in the per-device status conditions with a value of True to proceed with binding the pod to the node while scheduling the pod.  The maximum number of binding conditions is 4.  The conditions must be a valid condition type string.  This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.  # noqa: E501

        :param binding_conditions: The binding_conditions of this V1beta1BasicDevice.  # noqa: E501
        :type: list[str]
        """

        self._binding_conditions = binding_conditions

    @property
    def binding_failure_conditions(self):
        """Gets the binding_failure_conditions of this V1beta1BasicDevice.  # noqa: E501

        BindingFailureConditions defines the conditions for binding failure. They may be set in the per-device status conditions. If any is true, a binding failure occurred.  The maximum number of binding failure conditions is 4.  The conditions must be a valid condition type string.  This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.  # noqa: E501

        :return: The binding_failure_conditions of this V1beta1BasicDevice.  # noqa: E501
        :rtype: list[str]
        """
        return self._binding_failure_conditions

    @binding_failure_conditions.setter
    def binding_failure_conditions(self, binding_failure_conditions):
        """Sets the binding_failure_conditions of this V1beta1BasicDevice.

        BindingFailureConditions defines the conditions for binding failure. They may be set in the per-device status conditions. If any is true, a binding failure occurred.  The maximum number of binding failure conditions is 4.  The conditions must be a valid condition type string.  This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.  # noqa: E501

        :param binding_failure_conditions: The binding_failure_conditions of this V1beta1BasicDevice.  # noqa: E501
        :type: list[str]
        """

        self._binding_failure_conditions = binding_failure_conditions

    @property
    def binds_to_node(self):
        """Gets the binds_to_node of this V1beta1BasicDevice.  # noqa: E501

        BindsToNode indicates if the usage of an allocation involving this device has to be limited to exactly the node that was chosen when allocating the claim. If set to true, the scheduler will set the ResourceClaim.Status.Allocation.NodeSelector to match the node where the allocation was made.  This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.  # noqa: E501

        :return: The binds_to_node of this V1beta1BasicDevice.  # noqa: E501
        :rtype: bool
        """
        return self._binds_to_node

    @binds_to_node.setter
    def binds_to_node(self, binds_to_node):
        """Sets the binds_to_node of this V1beta1BasicDevice.

        BindsToNode indicates if the usage of an allocation involving this device has to be limited to exactly the node that was chosen when allocating the claim. If set to true, the scheduler will set the ResourceClaim.Status.Allocation.NodeSelector to match the node where the allocation was made.  This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gates.  # noqa: E501

        :param binds_to_node: The binds_to_node of this V1beta1BasicDevice.  # noqa: E501
        :type: bool
        """

        self._binds_to_node = binds_to_node

    @property
    def capacity(self):
        """Gets the capacity of this V1beta1BasicDevice.  # noqa: E501

        Capacity defines the set of capacities for this device. The name of each capacity must be unique in that set.  The maximum number of attributes and capacities combined is 32.  # noqa: E501

        :return: The capacity of this V1beta1BasicDevice.  # noqa: E501
        :rtype: dict(str, V1beta1DeviceCapacity)
        """
        return self._capacity

    @capacity.setter
    def capacity(self, capacity):
        """Sets the capacity of this V1beta1BasicDevice.

        Capacity defines the set of capacities for this device. The name of each capacity must be unique in that set.  The maximum number of attributes and capacities combined is 32.  # noqa: E501

        :param capacity: The capacity of this V1beta1BasicDevice.  # noqa: E501
        :type: dict(str, V1beta1DeviceCapacity)
        """

        self._capacity = capacity

    @property
    def consumes_counters(self):
        """Gets the consumes_counters of this V1beta1BasicDevice.  # noqa: E501

        ConsumesCounters defines a list of references to sharedCounters and the set of counters that the device will consume from those counter sets.  There can only be a single entry per counterSet.  The maximum number of device counter consumptions per device is 2.  # noqa: E501

        :return: The consumes_counters of this V1beta1BasicDevice.  # noqa: E501
        :rtype: list[V1beta1DeviceCounterConsumption]
        """
        return self._consumes_counters

    @consumes_counters.setter
    def consumes_counters(self, consumes_counters):
        """Sets the consumes_counters of this V1beta1BasicDevice.

        ConsumesCounters defines a list of references to sharedCounters and the set of counters that the device will consume from those counter sets.  There can only be a single entry per counterSet.  The maximum number of device counter consumptions per device is 2.  # noqa: E501

        :param consumes_counters: The consumes_counters of this V1beta1BasicDevice.  # noqa: E501
        :type: list[V1beta1DeviceCounterConsumption]
        """

        self._consumes_counters = consumes_counters

    @property
    def node_name(self):
        """Gets the node_name of this V1beta1BasicDevice.  # noqa: E501

        NodeName identifies the node where the device is available.  Must only be set if Spec.PerDeviceNodeSelection is set to true. At most one of NodeName, NodeSelector and AllNodes can be set.  # noqa: E501

        :return: The node_name of this V1beta1BasicDevice.  # noqa: E501
        :rtype: str
        """
        return self._node_name

    @node_name.setter
    def node_name(self, node_name):
        """Sets the node_name of this V1beta1BasicDevice.

        NodeName identifies the node where the device is available.  Must only be set if Spec.PerDeviceNodeSelection is set to true. At most one of NodeName, NodeSelector and AllNodes can be set.  # noqa: E501

        :param node_name: The node_name of this V1beta1BasicDevice.  # noqa: E501
        :type: str
        """

        self._node_name = node_name

    @property
    def node_selector(self):
        """Gets the node_selector of this V1beta1BasicDevice.  # noqa: E501


        :return: The node_selector of this V1beta1BasicDevice.  # noqa: E501
        :rtype: V1NodeSelector
        """
        return self._node_selector

    @node_selector.setter
    def node_selector(self, node_selector):
        """Sets the node_selector of this V1beta1BasicDevice.


        :param node_selector: The node_selector of this V1beta1BasicDevice.  # noqa: E501
        :type: V1NodeSelector
        """

        self._node_selector = node_selector

    @property
    def taints(self):
        """Gets the taints of this V1beta1BasicDevice.  # noqa: E501

        If specified, these are the driver-defined taints.  The maximum number of taints is 16. If taints are set for any device in a ResourceSlice, then the maximum number of allowed devices per ResourceSlice is 64 instead of 128.  This is an alpha field and requires enabling the DRADeviceTaints feature gate.  # noqa: E501

        :return: The taints of this V1beta1BasicDevice.  # noqa: E501
        :rtype: list[V1beta1DeviceTaint]
        """
        return self._taints

    @taints.setter
    def taints(self, taints):
        """Sets the taints of this V1beta1BasicDevice.

        If specified, these are the driver-defined taints.  The maximum number of taints is 16. If taints are set for any device in a ResourceSlice, then the maximum number of allowed devices per ResourceSlice is 64 instead of 128.  This is an alpha field and requires enabling the DRADeviceTaints feature gate.  # noqa: E501

        :param taints: The taints of this V1beta1BasicDevice.  # noqa: E501
        :type: list[V1beta1DeviceTaint]
        """

        self._taints = taints

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1beta1BasicDevice):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1beta1BasicDevice):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: release-1.35
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kubernetes.client.configuration import Configuration


class VersionInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'build_date': 'str',
        'compiler': 'str',
        'emulation_major': 'str',
        'emulation_minor': 'str',
        'git_commit': 'str',
        'git_tree_state': 'str',
        'git_version': 'str',
        'go_version': 'str',
        'major': 'str',
        'min_compatibility_major': 'str',
        'min_compatibility_minor': 'str',
        'minor': 'str',
        'platform': 'str'
    }

    attribute_map = {
        'build_date': 'buildDate',
        'compiler': 'compiler',
        'emulation_major': 'emulationMajor',
        'emulation_minor': 'emulationMinor',
        'git_commit': 'gitCommit',
        'git_tree_state': 'gitTreeState',
        'git_version': 'gitVersion',
        'go_version': 'goVersion',
        'major': 'major',
        'min_compatibility_major': 'minCompatibilityMajor',
        'min_compatibility_minor': 'minCompatibilityMinor',
        'minor': 'minor',
        'platform': 'platform'
    }

    def __init__(self, build_date=None, compiler=None, emulation_major=None, emulation_minor=None, git_commit=None, git_tree_state=None, git_version=None, go_version=None, major=None, min_compatibility_major=None, min_compatibility_minor=None, minor=None, platform=None, local_vars_configuration=None):  # noqa: E501
        """VersionInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._build_date = None
        self._compiler = None
        self._emulation_major = None
        self._emulation_minor = None
        self._git_commit = None
        self._git_tree_state = None
        self._git_version = None
        self._go_version = None
        self._major = None
        self._min_compatibility_major = None
        self._min_compatibility_minor = None
        self._minor = None
        self._platform = None
        self.discriminator = None

        self.build_date = build_date
        self.compiler = compiler
        if emulation_major is not None:
            self.emulation_major = emulation_major
        if emulation_minor is not None:
            self.emulation_minor = emulation_minor
        self.git_commit = git_commit
        self.git_tree_state = git_tree_state
        self.git_version = git_version
        self.go_version = go_version
        self.major = major
        if min_compatibility_major is not None:
            self.min_compatibility_major = min_compatibility_major
        if min_compatibility_minor is not None:
            self.min_compatibility_minor = min_compatibility_minor
        self.minor = minor
        self.platform = platform

    @property
    def build_date(self):
        """Gets the build_date of this VersionInfo.  # noqa: E501


        :return: The build_date of this VersionInfo.  # noqa: E501
        :rtype: str
        """
        return self._build_date

    @build_date.setter
    def build_date(self, build_date):
        """Sets the build_date of this VersionInfo.


        :param build_date: The build_date of this VersionInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and build_date is None:  # noqa: E501
            raise ValueError("Invalid value for `build_date`, must not be `None`")  # noqa: E501

        self._build_date = build_date

    @property
    def compiler(self):
        """Gets the compiler of this VersionInfo.  # noqa: E501


        :return: The compiler of this VersionInfo.  # noqa: E501
        :rtype: str
        """
        return self._compiler

    @compiler.setter
    def compiler(self, compiler):
        """Sets the compiler of this VersionInfo.


        :param compiler: The compiler of this VersionInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and compiler is None:  # noqa: E501
            raise ValueError("Invalid value for `compiler`, must not be `None`")  # noqa: E501

        self._compiler = compiler

    @property
    def emulation_major(self):
        """Gets the emulation_major of this VersionInfo.  # noqa: E501

        EmulationMajor is the major version of the emulation version  # noqa: E501

        :return: The emulation_major of this VersionInfo.  # noqa: E501
        :rtype: str
        """
        return self._emulation_major

    @emulation_major.setter
    def emulation_major(self, emulation_major):
        """Sets the emulation_major of this VersionInfo.

        EmulationMajor is the major version of the emulation version  # noqa: E501

        :param emulation_major: The emulation_major of this VersionInfo.  # noqa: E501
        :type: str
        """

        self._emulation_major = emulation_major

    @property
    def emulation_minor(self):
        """Gets the emulation_minor of this VersionInfo.  # noqa: E501

        EmulationMinor is the minor version of the emulation version  # noqa: E501

        :return: The emulation_minor of this VersionInfo.  # noqa: E501
        :rtype: str
        """
        return self._emulation_minor

    @emulation_minor.setter
    def emulation_minor(self, emulation_minor):
        """Sets the emulation_minor of this VersionInfo.

        EmulationMinor is the minor version of the emulation version  # noqa: E501

        :param emulation_minor: The emulation_minor of this VersionInfo.  # noqa: E501
        :type: str
        """

        self._emulation_minor = emulation_minor

    @property
    def git_commit(self):
        """Gets the git_commit of this VersionInfo.  # noqa: E501


        :return: The git_commit of this VersionInfo.  # noqa: E501
        :rtype: str
        """
        return self._git_commit

    @git_commit.setter
    def git_commit(self, git_commit):
        """Sets the git_commit of this VersionInfo.


        :param git_commit: The git_commit of this VersionInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and git_commit is None:  # noqa: E501
            raise ValueError("Invalid value for `git_commit`, must not be `None`")  # noqa: E501

        self._git_commit = git_commit

    @property
    def git_tree_state(self):
        """Gets the git_tree_state of this VersionInfo.  # noqa: E501


        :return: The git_tree_state of this VersionInfo.  # noqa: E501
        :rtype: str
        """
        return self._git_tree_state

    @git_tree_state.setter
    def git_tree_state(self, git_tree_state):
        """Sets the git_tree_state of this VersionInfo.


        :param git_tree_state: The git_tree_state of this VersionInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and git_tree_state is None:  # noqa: E501
            raise ValueError("Invalid value for `git_tree_state`, must not be `None`")  # noqa: E501

        self._git_tree_state = git_tree_state

    @property
    def git_version(self):
        """Gets the git_version of this VersionInfo.  # noqa: E501


        :return: The git_version of this VersionInfo.  # noqa: E501
        :rtype: str
        """
        return self._git_version

    @git_version.setter
    def git_version(self, git_version):
        """Sets the git_version of this VersionInfo.


        :param git_version: The git_version of this VersionInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and git_version is None:  # noqa: E501
            raise ValueError("Invalid value for `git_version`, must not be `None`")  # noqa: E501

        self._git_version = git_version

    @property
    def go_version(self):
        """Gets the go_version of this VersionInfo.  # noqa: E501


        :return: The go_version of this VersionInfo.  # noqa: E501
        :rtype: str
        """
        return self._go_version

    @go_version.setter
    def go_version(self, go_version):
        """Sets the go_version of this VersionInfo.


        :param go_version: The go_version of this VersionInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and go_version is None:  # noqa: E501
            raise ValueError("Invalid value for `go_version`, must not be `None`")  # noqa: E501

        self._go_version = go_version

    @property
    def major(self):
        """Gets the major of this VersionInfo.  # noqa: E501

        Major is the major version of the binary version  # noqa: E501

        :return: The major of this VersionInfo.  # noqa: E501
        :rtype: str
        """
        return self._major

    @major.setter
    def major(self, major):
        """Sets the major of this VersionInfo.

        Major is the major version of the binary version  # noqa: E501

        :param major: The major of this VersionInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and major is None:  # noqa: E501
            raise ValueError("Invalid value for `major`, must not be `None`")  # noqa: E501

        self._major = major

    @property
    def min_compatibility_major(self):
        """Gets the min_compatibility_major of this VersionInfo.  # noqa: E501

        MinCompatibilityMajor is the major version of the minimum compatibility version  # noqa: E501

        :return: The min_compatibility_major of this VersionInfo.  # noqa: E501
        :rtype: str
        """
        return self._min_compatibility_major

    @min_compatibility_major.setter
    def min_compatibility_major(self, min_compatibility_major):
        """Sets the min_compatibility_major of this VersionInfo.

        MinCompatibilityMajor is the major version of the minimum compatibility version  # noqa: E501

        :param min_compatibility_major: The min_compatibility_major of this VersionInfo.  # noqa: E501
        :type: str
        """

        self._min_compatibility_major = min_compatibility_major

    @property
    def min_compatibility_minor(self):
        """Gets the min_compatibility_minor of this VersionInfo.  # noqa: E501

        MinCompatibilityMinor is the minor version of the minimum compatibility version  # noqa: E501

        :return: The min_compatibility_minor of this VersionInfo.  # noqa: E501
        :rtype: str
        """
        return self._min_compatibility_minor

    @min_compatibility_minor.setter
    def min_compatibility_minor(self, min_compatibility_minor):
        """Sets the min_compatibility_minor of this VersionInfo.

        MinCompatibilityMinor is the minor version of the minimum compatibility version  # noqa: E501

        :param min_compatibility_minor: The min_compatibility_minor of this VersionInfo.  # noqa: E501
        :type: str
        """

        self._min_compatibility_minor = min_compatibility_minor

    @property
    def minor(self):
        """Gets the minor of this VersionInfo.  # noqa: E501

        Minor is the minor version of the binary version  # noqa: E501

        :return: The minor of this VersionInfo.  # noqa: E501
        :rtype: str
        """
        return self._minor

    @minor.setter
    def minor(self, minor):
        """Sets the minor of this VersionInfo.

        Minor is the minor version of the binary version  # noqa: E501

        :param minor: The minor of this VersionInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and minor is None:  # noqa: E501
            raise ValueError("Invalid value for `minor`, must not be `None`")  # noqa: E501

        self._minor = minor

    @property
    def platform(self):
        """Gets the platform of this VersionInfo.  # noqa: E501


        :return: The platform of this VersionInfo.  # noqa: E501
        :rtype: str
        """
        return self._platform

    @platform.setter
    def platform(self, platform):
        """Sets the platform of this VersionInfo.


        :param platform: The platform of this VersionInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and platform is None:  # noqa: E501
            raise ValueError("Invalid value for `platform`, must not be `None`")  # noqa: E501

        self._platform = platform

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, VersionInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, VersionInfo):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: release-1.35
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kubernetes.client.configuration import Configuration


class V1beta1PodCertificateRequestStatus(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'begin_refresh_at': 'datetime',
        'certificate_chain': 'str',
        'conditions': 'list[V1Condition]',
        'not_after': 'datetime',
        'not_before': 'datetime'
    }

    attribute_map = {
        'begin_refresh_at': 'beginRefreshAt',
        'certificate_chain': 'certificateChain',
        'conditions': 'conditions',
        'not_after': 'notAfter',
        'not_before': 'notBefore'
    }

    def __init__(self, begin_refresh_at=None, certificate_chain=None, conditions=None, not_after=None, not_before=None, local_vars_configuration=None):  # noqa: E501
        """V1beta1PodCertificateRequestStatus - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._begin_refresh_at = None
        self._certificate_chain = None
        self._conditions = None
        self._not_after = None
        self._not_before = None
        self.discriminator = None

        if begin_refresh_at is not None:
            self.begin_refresh_at = begin_refresh_at
        if certificate_chain is not None:
            self.certificate_chain = certificate_chain
        if conditions is not None:
            self.conditions = conditions
        if not_after is not None:
            self.not_after = not_after
        if not_before is not None:
            self.not_before = not_before

    @property
    def begin_refresh_at(self):
        """Gets the begin_refresh_at of this V1beta1PodCertificateRequestStatus.  # noqa: E501

        beginRefreshAt is the time at which the kubelet should begin trying to refresh the certificate.  This field is set via the /status subresource, and must be set at the same time as certificateChain.  Once populated, this field is immutable.  This field is only a hint.  Kubelet may start refreshing before or after this time if necessary.  # noqa: E501

        :return: The begin_refresh_at of this V1beta1PodCertificateRequestStatus.  # noqa: E501
        :rtype: datetime
        """
        return self._begin_refresh_at

    @begin_refresh_at.setter
    def begin_refresh_at(self, begin_refresh_at):
        """Sets the begin_refresh_at of this V1beta1PodCertificateRequestStatus.

        beginRefreshAt is the time at which the kubelet should begin trying to refresh the certificate.  This field is set via the /status subresource, and must be set at the same time as certificateChain.  Once populated, this field is immutable.  This field is only a hint.  Kubelet may start refreshing before or after this time if necessary.  # noqa: E501

        :param begin_refresh_at: The begin_refresh_at of this V1beta1PodCertificateRequestStatus.  # noqa: E501
        :type: datetime
        """

        self._begin_refresh_at = begin_refresh_at

    @property
    def certificate_chain(self):
        """Gets the certificate_chain of this V1beta1PodCertificateRequestStatus.  # noqa: E501

        certificateChain is populated with an issued certificate by the signer. This field is set via the /status subresource. Once populated, this field is immutable.  If the certificate signing request is denied, a condition of type \"Denied\" is added and this field remains empty. If the signer cannot issue the certificate, a condition of type \"Failed\" is added and this field remains empty.  Validation requirements:  1. certificateChain must consist of one or more PEM-formatted certificates.  2. Each entry must be a valid PEM-wrapped, DER-encoded ASN.1 Certificate as     described in section 4 of RFC5280.  If more than one block is present, and the definition of the requested spec.signerName does not indicate otherwise, the first block is the issued certificate, and subsequent blocks should be treated as intermediate certificates and presented in TLS handshakes.  When projecting the chain into a pod volume, kubelet will drop any data in-between the PEM blocks, as well as any PEM block headers.  # noqa: E501

        :return: The certificate_chain of this V1beta1PodCertificateRequestStatus.  # noqa: E501
        :rtype: str
        """
        return self._certificate_chain

    @certificate_chain.setter
    def certificate_chain(self, certificate_chain):
        """Sets the certificate_chain of this V1beta1PodCertificateRequestStatus.

        certificateChain is populated with an issued certificate by the signer. This field is set via the /status subresource. Once populated, this field is immutable.  If the certificate signing request is denied, a condition of type \"Denied\" is added and this field remains empty. If the signer cannot issue the certificate, a condition of type \"Failed\" is added and this field remains empty.  Validation requirements:  1. certificateChain must consist of one or more PEM-formatted certificates.  2. Each entry must be a valid PEM-wrapped, DER-encoded ASN.1 Certificate as     described in section 4 of RFC5280.  If more than one block is present, and the definition of the requested spec.signerName does not indicate otherwise, the first block is the issued certificate, and subsequent blocks should be treated as intermediate certificates and presented in TLS handshakes.  When projecting the chain into a pod volume, kubelet will drop any data in-between the PEM blocks, as well as any PEM block headers.  # noqa: E501

        :param certificate_chain: The certificate_chain of this V1beta1PodCertificateRequestStatus.  # noqa: E501
        :type: str
        """

        self._certificate_chain = certificate_chain

    @property
    def conditions(self):
        """Gets the conditions of this V1beta1PodCertificateRequestStatus.  # noqa: E501

        conditions applied to the request.  The types \"Issued\", \"Denied\", and \"Failed\" have special handling.  At most one of these conditions may be present, and they must have status \"True\".  If the request is denied with `Reason=UnsupportedKeyType`, the signer may suggest a key type that will work in the message field.  # noqa: E501

        :return: The conditions of this V1beta1PodCertificateRequestStatus.  # noqa: E501
        :rtype: list[V1Condition]
        """
        return self._conditions

    @conditions.setter
    def conditions(self, conditions):
        """Sets the conditions of this V1beta1PodCertificateRequestStatus.

        conditions applied to the request.  The types \"Issued\", \"Denied\", and \"Failed\" have special handling.  At most one of these conditions may be present, and they must have status \"True\".  If the request is denied with `Reason=UnsupportedKeyType`, the signer may suggest a key type that will work in the message field.  # noqa: E501

        :param conditions: The conditions of this V1beta1PodCertificateRequestStatus.  # noqa: E501
        :type: list[V1Condition]
        """

        self._conditions = conditions

    @property
    def not_after(self):
        """Gets the not_after of this V1beta1PodCertificateRequestStatus.  # noqa: E501

        notAfter is the time at which the certificate expires.  The value must be the same as the notAfter value in the leaf certificate in certificateChain.  This field is set via the /status subresource.  Once populated, it is immutable.  The signer must set this field at the same time it sets certificateChain.  # noqa: E501

        :return: The not_after of this V1beta1PodCertificateRequestStatus.  # noqa: E501
        :rtype: datetime
        """
        return self._not_after

    @not_after.setter
    def not_after(self, not_after):
        """Sets the not_after of this V1beta1PodCertificateRequestStatus.

        notAfter is the time at which the certificate expires.  The value must be the same as the notAfter value in the leaf certificate in certificateChain.  This field is set via the /status subresource.  Once populated, it is immutable.  The signer must set this field at the same time it sets certificateChain.  # noqa: E501

        :param not_after: The not_after of this V1beta1PodCertificateRequestStatus.  # noqa: E501
        :type: datetime
        """

        self._not_after = not_after

    @property
    def not_before(self):
        """Gets the not_before of this V1beta1PodCertificateRequestStatus.  # noqa: E501

        notBefore is the time at which the certificate becomes valid.  The value must be the same as the notBefore value in the leaf certificate in certificateChain.  This field is set via the /status subresource.  Once populated, it is immutable. The signer must set this field at the same time it sets certificateChain.  # noqa: E501

        :return: The not_before of this V1beta1PodCertificateRequestStatus.  # noqa: E501
        :rtype: datetime
        """
        return self._not_before

    @not_before.setter
    def not_before(self, not_before):
        """Sets the not_before of this V1beta1PodCertificateRequestStatus.

        notBefore is the time at which the certificate becomes valid.  The value must be the same as the notBefore value in the leaf certificate in certificateChain.  This field is set via the /status subresource.  Once populated, it is immutable. The signer must set this field at the same time it sets certificateChain.  # noqa: E501

        :param not_before: The not_before of this V1beta1PodCertificateRequestStatus.  # noqa: E501
        :type: datetime
        """

        self._not_before = not_before

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1beta1PodCertificateRequestStatus):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1beta1PodCertificateRequestStatus):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: release-1.35
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kubernetes.client.configuration import Configuration


class V1beta2AllocationResult(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'allocation_timestamp': 'datetime',
        'devices': 'V1beta2DeviceAllocationResult',
        'node_selector': 'V1NodeSelector'
    }

    attribute_map = {
        'allocation_timestamp': 'allocationTimestamp',
        'devices': 'devices',
        'node_selector': 'nodeSelector'
    }

    def __init__(self, allocation_timestamp=None, devices=None, node_selector=None, local_vars_configuration=None):  # noqa: E501
        """V1beta2AllocationResult - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._allocation_timestamp = None
        self._devices = None
        self._node_selector = None
        self.discriminator = None

        if allocation_timestamp is not None:
            self.allocation_timestamp = allocation_timestamp
        if devices is not None:
            self.devices = devices
        if node_selector is not None:
            self.node_selector = node_selector

    @property
    def allocation_timestamp(self):
        """Gets the allocation_timestamp of this V1beta2AllocationResult.  # noqa: E501

        AllocationTimestamp stores the time when the resources were allocated. This field is not guaranteed to be set, in which case that time is unknown.  This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gate.  # noqa: E501

        :return: The allocation_timestamp of this V1beta2AllocationResult.  # noqa: E501
        :rtype: datetime
        """
        return self._allocation_timestamp

    @allocation_timestamp.setter
    def allocation_timestamp(self, allocation_timestamp):
        """Sets the allocation_timestamp of this V1beta2AllocationResult.

        AllocationTimestamp stores the time when the resources were allocated. This field is not guaranteed to be set, in which case that time is unknown.  This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gate.  # noqa: E501

        :param allocation_timestamp: The allocation_timestamp of this V1beta2AllocationResult.  # noqa: E501
        :type: datetime
        """

        self._allocation_timestamp = allocation_timestamp

    @property
    def devices(self):
        """Gets the devices of this V1beta2AllocationResult.  # noqa: E501


        :return: The devices of this V1beta2AllocationResult.  # noqa: E501
        :rtype: V1beta2DeviceAllocationResult
        """
        return self._devices

    @devices.setter
    def devices(self, devices):
        """Sets the devices of this V1beta2AllocationResult.


        :param devices: The devices of this V1beta2AllocationResult.  # noqa: E501
        :type: V1beta2DeviceAllocationResult
        """

        self._devices = devices

    @property
    def node_selector(self):
        """Gets the node_selector of this V1beta2AllocationResult.  # noqa: E501


        :return: The node_selector of this V1beta2AllocationResult.  # noqa: E501
        :rtype: V1NodeSelector
        """
        return self._node_selector

    @node_selector.setter
    def node_selector(self, node_selector):
        """Sets the node_selector of this V1beta2AllocationResult.


        :param node_selector: The node_selector of this V1beta2AllocationResult.  # noqa: E501
        :type: V1NodeSelector
        """

        self._node_selector = node_selector

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1beta2AllocationResult):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1beta2AllocationResult):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: release-1.35
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kubernetes.client.configuration import Configuration


class V1ScaleStatus(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'replicas': 'int',
        'selector': 'str'
    }

    attribute_map = {
        'replicas': 'replicas',
        'selector': 'selector'
    }

    def __init__(self, replicas=None, selector=None, local_vars_configuration=None):  # noqa: E501
        """V1ScaleStatus - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._replicas = None
        self._selector = None
        self.discriminator = None

        self.replicas = replicas
        if selector is not None:
            self.selector = selector

    @property
    def replicas(self):
        """Gets the replicas of this V1ScaleStatus.  # noqa: E501

        replicas is the actual number of observed instances of the scaled object.  # noqa: E501

        :return: The replicas of this V1ScaleStatus.  # noqa: E501
        :rtype: int
        """
        return self._replicas

    @replicas.setter
    def replicas(self, replicas):
        """Sets the replicas of this V1ScaleStatus.

        replicas is the actual number of observed instances of the scaled object.  # noqa: E501

        :param replicas: The replicas of this V1ScaleStatus.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and replicas is None:  # noqa: E501
            raise ValueError("Invalid value for `replicas`, must not be `None`")  # noqa: E501

        self._replicas = replicas

    @property
    def selector(self):
        """Gets the selector of this V1ScaleStatus.  # noqa: E501

        selector is the label query over pods that should match the replicas count. This is same as the label selector but in the string format to avoid introspection by clients. The string will be in the same format as the query-param syntax. More info about label selectors: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/  # noqa: E501

        :return: The selector of this V1ScaleStatus.  # noqa: E501
        :rtype: str
        """
        return self._selector

    @selector.setter
    def selector(self, selector):
        """Sets the selector of this V1ScaleStatus.

        selector is the label query over pods that should match the replicas count. This is same as the label selector but in the string format to avoid introspection by clients. The string will be in the same format as the query-param syntax. More info about label selectors: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/  # noqa: E501

        :param selector: The selector of this V1ScaleStatus.  # noqa: E501
        :type: str
        """

        self._selector = selector

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1ScaleStatus):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1ScaleStatus):
            return True

        return self.to_dict() != other.to_dict()
